unifind<-function(x, y, PV, trim=0.05, draw.spectra=T, draw.p=F, ...){

#  Authors: Luca Laghi (l.laghi@unibo.it), Chenglin Zhu, 1Department of Agri-Food Science and Technology, University of Bologna, Bologna, Italy

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/




# check of the input variables
if (class(x)!="numeric") stop("unifind warning: x is expected to be a vector with the spectra ppms")
if (class(y)!="matrix") stop("unifind warning: y is expected to be a matrix with the spectra intensities")
PV1<-as.vector(PV)

L<-length(x)
if (dim(y)[2]!=L) stop ("unifind warning: y columns should be equal to x length")
if (length(PV1)!=L) stop ("unifind warning: PV1 and x length are expected to be identical")

PV_TF<-rep(F, L)
PV2TF<-rep(F, L)




#### START OF Part 1: Creation of the vector "signals.points", here called PV_TF
# Unifind runs from upfield to downfield. When it finds a point with PV1 below the trim value it reconstructs the right part of the pseudo-signal
for (i in 1:c(L-1)){
if(is.na(PV1[i])) next() # Decision 1
# if(PV1[i] >= trim & PV_TF[i]==F) next() # Decision 1
if(PV1[i] >= trim) next() # Decision 2
if(PV1[i]<trim) PV_TF[i]<-T # Process 1
if(is.na(PV1[i+1])) next() # Decision 3
if(PV1[i+1]<trim) next() # Decision 4
if(PV1[i+1]>PV1[i]) PV_TF[i+1]<-T # Decision 5 Process 2
}
# Unifind runs from downfield to upfield. When it finds a point with PV1 below the trim value it reconstructs the left part of the pseudo-signal
for (i in L:2){
if(is.na(PV1[i])) next()
if(PV1[i]>trim & PV_TF[i]==F) next()
if(PV1[i]<trim) PV_TF[i]<-T
if(is.na(PV1[i-1])) next()
if(PV1[i-1]<trim) next()
if(PV1[i-1]>PV1[i]) PV_TF[i-1]<-T
}
#### END OF Creation of the vector "signals.points", here called PV_TF





#### START OF Part 2: Creation of the vector signals.extremes, here simply called "signals", by merging contiguous signals
for (i in 2:c(L-1)){
if (PV_TF[i] & PV_TF[i+1]==F) PV2TF[i]<-T
if (PV_TF[i] & PV_TF[i-1]==F) PV2TF[i]<-T
}
# If the last signal hasn't got any closure because at the end of the spectrum, I force its closure
if (sum(PV2TF)%%2!=0) PV2TF[L]<-T
# A two columns matrix is created with initial and final ppm value for each identified signal 
signals<-matrix(data=x[PV2TF], ncol=2, byrow=T)
## END OF Creation of the vector signals.extremes by merging contiguous signals




#### START OF Part 3: Plotting procedure
Lsig<-dim(signals)[1]
if (draw.spectra){
	matplot(x, t(y), type="l", lty=1, xlab="Chemical shift (ppm)", yaxt="n", ylab="", ...) # Drawing of the spectra
	YL<-par("usr")[3:4]
if (draw.p){
	abline(h=trim/max(PV1)*max(YL), col=3, lwd=2, lty=2)
	axis(2, at=trim/max(PV1)*max(YL), labels="trim val.", tick=F, las=1, padj=0, col.axis=3)
	lines(x, PV1/max(PV1)*max(YL), col=3, lwd=2)
	}
	for(i in 1:Lsig){ # Gray shadow on the signals
	polygon(x=c(signals[i,1],signals[i,1],signals[i,2],signals[i,2]), y=c(min(YL),max(YL),max(YL),min(YL))*0.99, border=1, lwd=2, col="#00000020")
}

}

# The output object is created
out<-list(signals.points=PV_TF, signals.extremes=signals)

return(out)
}
